local EnhancedModelScene = select(2, ...).self

local ActorMixin = {}
EnhancedModelScene.ActorMixin = ActorMixin


function ActorMixin:GetRelativeMatrix(reference)
	local inverse = reference:GetCenterMatrix():GetInverse()
	return EnhancedModelScene.Matrix.hmult(inverse, self:GetCenterMatrix())
end


function ActorMixin:GetScaledCenterOffset()
	local x, y, z = self:GetRawCenter()
	local s = self:GetScale()
	
	return s*x, s*y, s*z
end


function ActorMixin:GetAngles()
	return self:GetYaw(), self:GetPitch(), self:GetRoll()
end


function ActorMixin:GetRotationMatrix()
	-- TODO: maybe just return YPR and check if the matrix is actually needed
	return EnhancedModelScene.Matrix.YPR(self:GetYaw(), self:GetPitch(), self:GetRoll())
end


function ActorMixin:GetYPR()
	return self:GetYaw(), self:GetPitch(), self:GetRoll()
end


function ActorMixin:GetYPRDeg()
	return deg(self:GetYaw()), deg(self:GetPitch()), deg(self:GetRoll())
end


function ActorMixin:GetCenterMatrix()
	local m = self:GetRotationMatrix()
	m:Scale(self:GetScale())
	m:SetTranslation(self:GetScaledPosition())
	return m:AddTranslation(self:GetScaledCenterOffset())
end


function ActorMixin:SetOrientation(m)
	local yaw, pitch, roll = m:GetYPR()
	
	self:SetYaw(yaw)
	self:SetPitch(pitch)
	self:SetRoll(roll)
end


function ActorMixin:SetCenterTransform(m)
	local x, y, z = m:GetXYZ()
	self:SetCenterPosition(x, y, z)
	
	self:SetOrientation(m)
	
	self:SetScale(m:GetScale())
end



function ActorMixin:GetScaledPosition()
	local x, y, z = self:GetPosition()
	local s = self:GetScale()
	return s*x, s*y, s*z
end


ActorMixin.GetXYZ = ActorMixin.GetScaledPosition


function ActorMixin:SetScaledPosition(x, y, z)
	local s = self:GetScale()
	self:SetPosition(x/s, y/s, z/s)
end


function ActorMixin:SetCenterPosition(x, y, z)
	local cx, cy, cz = self:GetScaledCenterOffset()
	self:SetScaledPosition(x - cx, y - cy, z - cz)
end


function ActorMixin:GetRawCenter()
	return EnhancedModelScene:GetRawCenterFor(self:GetModelFileID(), self.animation)
end


function ActorMixin:GetCenterPosition()
	-- appears to be affected by scale but not orientation
	local x, y, z = self:GetScaledPosition()
	local s = self:GetScale()
	
	local a, b, c = self:GetRawCenter()
	x, y, z = x+s*a, y+s*b, z+s*c
	
	return x, y, z
end


function ActorMixin:SlideOnAxis(axis, distance)
	local x, y, z = self:GetRotationMatrix():GetBasisVector(axis)
	self:Slide(x, y, z, distance)
end


function ActorMixin:Slide(x, y, z, scale)
	local xi, yi, zi = self:GetScaledPosition()
	self:SetScaledPosition(xi + x * scale, yi + y * scale, zi + z * scale)
end


function ActorMixin:ConvertToTransform(relative_to)
end


function ActorMixin:GetKey()
	-- FIXME
	--return self.key
	
end


-- TODO: maybe add function to directly get axis unit vectors


function ActorMixin:AddPhasor(phasor)
	if not self.phasors then self.phasors = {} end
	tinsert(self.phasors, phasor)
end


function ActorMixin:NewPhasor(...)
	self:AddPhasor(EnhancedModelScene.Phasor.New(...))
end


function ActorMixin:UpdatePhasors(elapsed)
	if self.phasors then
		for i = 1, #self.phasors do
			self.phasors[i]:Update(elapsed)
			self.phasors[1]:Apply()
		end
	end
end
